@@ -23,28 +23,22 @@
 
 set -e
 
-PYTHON_VERSIONS=(
-   '2.7 cp27-cp27mu manylinux1'
-   '2.7 cp27-cp27m manylinux1'
-   '3.5 cp35-cp35m manylinux2014'
-   '3.6 cp36-cp36m manylinux2014'
-   '3.7 cp37-cp37m manylinux2014'
-   '3.8 cp38-cp38 manylinux2014'
-   '3.9 cp39-cp39 manylinux2014'
-)
+source python-versions.sh
 
 for line in "${PYTHON_VERSIONS[@]}"; do
     read -r -a PY <<< "$line"
     PYTHON_VERSION=${PY[0]}
     PYTHON_SPEC=${PY[1]}
     BASE_IMAGE=${PY[2]}
-    echo "--------- Build Docker image for $PYTHON_VERSION -- $PYTHON_SPEC"
+    ARCH=${PY[3]}
+    echo "--------- Build Docker image for $PYTHON_VERSION -- $PYTHON_SPEC -- $ARCH"
 
-    IMAGE_NAME=pulsar-build:manylinux-$PYTHON_SPEC
+    IMAGE_NAME=pulsar-build:$BASE_IMAGE-$PYTHON_SPEC-$ARCH
 
     docker build -t $IMAGE_NAME $BASE_IMAGE \
             --build-arg PYTHON_VERSION=$PYTHON_VERSION \
-            --build-arg PYTHON_SPEC=$PYTHON_SPEC
+            --build-arg PYTHON_SPEC=$PYTHON_SPEC \
+            --build-arg ARCH=$ARCH
 
     echo "==== Successfully built image $IMAGE_NAME"
 done
