@@ -23,9 +23,9 @@
 #include "ProducerImpl.h"
 #include "ReaderImpl.h"
 #include "PartitionedProducerImpl.h"
-#include "PartitionedConsumerImpl.h"
 #include "MultiTopicsConsumerImpl.h"
 #include "PatternMultiTopicsConsumerImpl.h"
+#include "TimeUtils.h"
 #include <pulsar/ConsoleLoggerFactory.h>
 #include <boost/algorithm/string/predicate.hpp>
 #include <sstream>
@@ -35,6 +35,7 @@
 #include <algorithm>
 #include <random>
 #include <mutex>
+#include <thread>
 #ifdef USE_LOG4CXX
 #include "Log4CxxLogger.h"
 #endif
@@ -183,9 +184,6 @@ void ClientImpl::handleCreateProducer(const Result result, const LookupDataResul
         producer->getProducerCreatedFuture().addListener(
             std::bind(&ClientImpl::handleProducerCreated, shared_from_this(), std::placeholders::_1,
                       std::placeholders::_2, callback, producer));
-        Lock lock(mutex_);
-        producers_.push_back(producer);
-        lock.unlock();
         producer->start();
     } else {
         LOG_ERROR("Error Checking/Getting Partition Metadata while creating producer on "
@@ -196,7 +194,14 @@ void ClientImpl::handleCreateProducer(const Result result, const LookupDataResul
 
 void ClientImpl::handleProducerCreated(Result result, ProducerImplBaseWeakPtr producerBaseWeakPtr,
                                        CreateProducerCallback callback, ProducerImplBasePtr producer) {
-    callback(result, Producer(producer));
+    if (result == ResultOk) {
+        Lock lock(mutex_);
+        producers_.push_back(producer);
+        lock.unlock();
+        callback(result, Producer(producer));
+    } else {
+        callback(result, {});
+    }
 }
 
 void ClientImpl::createReaderAsync(const std::string& topic, const MessageId& startMessageId,
@@ -239,10 +244,13 @@ void ClientImpl::handleReaderMetadataLookup(const Result result, const LookupDat
 
     ReaderImplPtr reader = std::make_shared<ReaderImpl>(shared_from_this(), topicName->toString(), conf,
                                                         getListenerExecutorProvider()->get(), callback);
-    reader->start(startMessageId);
-
-    Lock lock(mutex_);
-    consumers_.push_back(reader->getConsumer());
+    ConsumerImplBasePtr consumer = reader->getConsumer().lock();
+    auto self = shared_from_this();
+    reader->start(startMessageId, [this, self](const ConsumerImplBaseWeakPtr& weakConsumerPtr) {
+        Lock lock(mutex_);
+        consumers_.push_back(weakConsumerPtr);
+        lock.unlock();
+    });
 }
 
 void ClientImpl::subscribeWithRegexAsync(const std::string& regexPattern, const std::string& subscriptionName,
@@ -289,9 +297,6 @@ void ClientImpl::createPatternMultiTopicsConsumer(const Result result, const Nam
         consumer->getConsumerCreatedFuture().addListener(
             std::bind(&ClientImpl::handleConsumerCreated, shared_from_this(), std::placeholders::_1,
                       std::placeholders::_2, callback, consumer));
-        Lock lock(mutex_);
-        consumers_.push_back(consumer);
-        lock.unlock();
         consumer->start();
     } else {
         LOG_ERROR("Error Getting topicsOfNameSpace while createPatternMultiTopicsConsumer:  " << result);
@@ -315,6 +320,7 @@ void ClientImpl::subscribeAsync(const std::vector<std::string>& topics, const st
             return;
         }
     }
+    lock.unlock();
 
     if (topicNamePtr) {
         std::string randomName = generateRandomName();
@@ -329,8 +335,6 @@ void ClientImpl::subscribeAsync(const std::vector<std::string>& topics, const st
     consumer->getConsumerCreatedFuture().addListener(std::bind(&ClientImpl::handleConsumerCreated,
                                                                shared_from_this(), std::placeholders::_1,
                                                                std::placeholders::_2, callback, consumer));
-    consumers_.push_back(consumer);
-    lock.unlock();
     consumer->start();
 }
 
@@ -376,8 +380,9 @@ void ClientImpl::handleSubscribe(const Result result, const LookupDataResultPtr
                 callback(ResultInvalidConfiguration, Consumer());
                 return;
             }
-            consumer = std::make_shared<PartitionedConsumerImpl>(
-                shared_from_this(), subscriptionName, topicName, partitionMetadata->getPartitions(), conf);
+            consumer = std::make_shared<MultiTopicsConsumerImpl>(shared_from_this(), topicName,
+                                                                 partitionMetadata->getPartitions(),
+                                                                 subscriptionName, conf, lookupServicePtr_);
         } else {
             auto consumerImpl = std::make_shared<ConsumerImpl>(shared_from_this(), topicName->toString(),
                                                                subscriptionName, conf);
@@ -387,9 +392,6 @@ void ClientImpl::handleSubscribe(const Result result, const LookupDataResultPtr
         consumer->getConsumerCreatedFuture().addListener(
             std::bind(&ClientImpl::handleConsumerCreated, shared_from_this(), std::placeholders::_1,
                       std::placeholders::_2, callback, consumer));
-        Lock lock(mutex_);
-        consumers_.push_back(consumer);
-        lock.unlock();
         consumer->start();
     } else {
         LOG_ERROR("Error Checking/Getting Partition Metadata while Subscribing on " << topicName->toString()
@@ -400,7 +402,14 @@ void ClientImpl::handleSubscribe(const Result result, const LookupDataResultPtr
 
 void ClientImpl::handleConsumerCreated(Result result, ConsumerImplBaseWeakPtr consumerImplBaseWeakPtr,
                                        SubscribeCallback callback, ConsumerImplBasePtr consumer) {
-    callback(result, Consumer(consumer));
+    if (result == ResultOk) {
+        Lock lock(mutex_);
+        consumers_.push_back(consumer);
+        lock.unlock();
+        callback(result, Consumer(consumer));
+    } else {
+        callback(result, {});
+    }
 }
 
 Future<Result, ClientConnectionWeakPtr> ClientImpl::getConnection(const std::string& topic) {
@@ -534,17 +543,29 @@ void ClientImpl::handleClose(Result result, SharedInt numberOfOpenHandlers, Resu
     }
     if (*numberOfOpenHandlers == 0) {
         Lock lock(mutex_);
-        state_ = Closed;
-        lock.unlock();
+        if (state_ == Closed) {
+            LOG_DEBUG("Client is already shutting down, possible race condition in handleClose");
+            return;
+        } else {
+            state_ = Closed;
+            lock.unlock();
+        }
 
         LOG_DEBUG("Shutting down producers and consumers for client");
-        shutdown();
-        if (callback) {
-            if (closingError != ResultOk) {
-                LOG_DEBUG("Problem in closing client, could not close one or more consumers or producers");
+        // handleClose() is called in ExecutorService's event loop, while shutdown() tried to wait the event
+        // loop exits. So here we use another thread to call shutdown().
+        auto self = shared_from_this();
+        std::thread shutdownTask{[this, self, callback] {
+            shutdown();
+            if (callback) {
+                if (closingError != ResultOk) {
+                    LOG_DEBUG(
+                        "Problem in closing client, could not close one or more consumers or producers");
+                }
+                callback(closingError);
             }
-            callback(closingError);
-        }
+        }};
+        shutdownTask.detach();
     }
 }
 
@@ -580,11 +601,25 @@ void ClientImpl::shutdown() {
         return;
     }
     LOG_DEBUG("ConnectionPool is closed");
-    ioExecutorProvider_->close();
+
+    // 500ms as the timeout is long enough because ExecutorService::close calls io_service::stop() internally
+    // and waits until io_service::run() in another thread returns, which should be as soon as possible after
+    // stop() is called.
+    TimeoutProcessor<std::chrono::milliseconds> timeoutProcessor{500};
+
+    timeoutProcessor.tik();
+    ioExecutorProvider_->close(timeoutProcessor.getLeftTimeout());
+    timeoutProcessor.tok();
     LOG_DEBUG("ioExecutorProvider_ is closed");
-    listenerExecutorProvider_->close();
+
+    timeoutProcessor.tik();
+    listenerExecutorProvider_->close(timeoutProcessor.getLeftTimeout());
+    timeoutProcessor.tok();
     LOG_DEBUG("listenerExecutorProvider_ is closed");
-    partitionListenerExecutorProvider_->close();
+
+    timeoutProcessor.tik();
+    partitionListenerExecutorProvider_->close(timeoutProcessor.getLeftTimeout());
+    timeoutProcessor.tok();
     LOG_DEBUG("partitionListenerExecutorProvider_ is closed");
 }
 
