@@ -460,4 +460,24 @@ public class ConcurrentOpenLongPairRangeSetTest {
         gRangeConnected.add(lastRange);
         return gRangeConnected;
     }
+
+    @Test
+    public void testCardinality() {
+        ConcurrentOpenLongPairRangeSet<LongPair> set = new ConcurrentOpenLongPairRangeSet<>(consumer);
+        int v = set.cardinality(0, 0, Integer.MAX_VALUE, Integer.MAX_VALUE);
+        assertEquals(v, 0 );
+        set.addOpenClosed(1, 0, 1, 20);
+        set.addOpenClosed(1, 30, 1, 90);
+        set.addOpenClosed(2, 0, 3, 30);
+        v = set.cardinality(1, 0, 1, 100);
+        assertEquals(v, 80);
+        v = set.cardinality(1, 11, 1, 100);
+        assertEquals(v, 70);
+        v = set.cardinality(1, 0, 1, 90);
+        assertEquals(v, 80);
+        v = set.cardinality(1, 0, 1, 80);
+        assertEquals(v, 70);
+        v = set.cardinality(1, 0, 3, 30);
+        assertEquals(v, 80 + 31);
+    }
 }
